#ifndef PRIORITY_EXECUTOR__MULTITHREAD_PRIORITY_EXECUTOR_HPP_
#define PRIORITY_EXECUTOR__MULTITHREAD_PRIORITY_EXECUTOR_HPP_

#include <set>
#include <mutex>
#include <chrono>
#include <memory>
#include <thread>
#include <unordered_map>

#include "priority_executor/default_executor.hpp"
#include "priority_executor/priority_executor.hpp"

#include "rclcpp/executor.hpp"
#include "rclcpp/macros.hpp"
#include "rclcpp/memory_strategies.hpp"
#include "rclcpp/node.hpp"
#include "rclcpp/rate.hpp"
#include "rclcpp/utilities.hpp"
#include "rclcpp/visibility_control.hpp"

namespace priority_executor {

class MultithreadTimedExecutor : public TimedExecutor {
public:
    RCLCPP_PUBLIC
    explicit MultithreadTimedExecutor(
        rclcpp::ExecutorOptions const &options = rclcpp::ExecutorOptions(),
        std::string const &name = "unnamed executor",
        int number_of_threads = 2,
        std::chrono::nanoseconds next_exec_timeout = std::chrono::nanoseconds(-1));

    RCLCPP_PUBLIC
    virtual ~MultithreadTimedExecutor() = default;

    RCLCPP_PUBLIC size_t get_number_of_threads();
    RCLCPP_PUBLIC void spin() override;

protected:
    RCLCPP_PUBLIC void run(size_t _thread_number);

private:
    std::set<rclcpp::TimerBase::SharedPtr> scheduled_timers_;
    size_t number_of_threads_;
    std::chrono::nanoseconds next_exec_timeout_ = std::chrono::nanoseconds(-1);
    node_time_logger logger_;

    static std::unordered_map<MultithreadTimedExecutor*,
                             std::shared_ptr<rclcpp::detail::MutexTwoPriorities>>
        wait_mutex_set_;
    static std::mutex shared_wait_mutex_;
};

}  // namespace priority_executor

#endif  // PRIORITY_EXECUTOR__MULTITHREAD_PRIORITY_EXECUTOR_HPP_